`timescale 1ns / 1ps
`default_nettype none

module axi4_lite_if #(
   //A hasznlt cmbitek szma.
   parameter ADDR_BITS = 5
) (
   //rajel s reset.
   input  wire                 clk,            //Rendszerrajel
   input  wire                 rst,            //Aktv magas szinkron reset
   
   //AXI4-Lite rsi cm csatorna.
   input  wire [ADDR_BITS-1:0] s_axi_awaddr,
   input  wire                 s_axi_awvalid,
   output wire                 s_axi_awready,
   
   //AXI4-Lite rsi adat csatorna.
   input  wire [31:0]          s_axi_wdata,
   input  wire [3:0]           s_axi_wstrb,
   input  wire                 s_axi_wvalid,
   output wire                 s_axi_wready,
   
   //AXI4-Lite rsi vlasz csatorna.
   output wire [1:0]           s_axi_bresp,
   output wire                 s_axi_bvalid,
   input  wire                 s_axi_bready,
   
   //AXI4-Lite olvassi cm csatorna.
   input  wire [ADDR_BITS-1:0] s_axi_araddr,
   input  wire                 s_axi_arvalid,
   output wire                 s_axi_arready,
   
   //AXI4-Lite olvassi adat csatorna.
   output reg  [31:0]          s_axi_rdata,
   output wire [1:0]           s_axi_rresp,
   output wire                 s_axi_rvalid,
   input  wire                 s_axi_rready,
   
   //Regiszter rsi interfsz.
   output reg  [4:0]           wr_addr,        //rsi cm
   output wire                 wr_en,          //rs engedlyez jel
   output reg  [31:0]          wr_data,        //rsi adat
   output reg  [3:0]           wr_strb,        //Bjt engedlyez jelek
    
   //Regiszter olvassi interfsz.
   output reg  [4:0]           rd_addr,        //Olvassi cm
   output wire                 rd_en,          //Olvass engedlyez jel
   input  wire [31:0]          rd_data         //Olvassi adat
);

//******************************************************************************
//* rsi llapotgp.                                                          *
//******************************************************************************
localparam WR_ADDR_WAIT = 2'd0;
localparam WR_DATA_WAIT = 2'd1;
localparam WR_EXECUTE   = 2'd2;
localparam WR_RESPONSE  = 2'd3;

reg [1:0] wr_state;

always @(posedge clk)
begin
   if (rst)
      wr_state <= WR_ADDR_WAIT;
   else
      case (wr_state)
         //Vraozs az rsi cmre.
         WR_ADDR_WAIT: if (s_axi_awvalid)
                       begin
                          wr_state <= WR_DATA_WAIT;
                          wr_addr  <= s_axi_awaddr;
                       end
                       else
                          wr_state <= WR_ADDR_WAIT;

         //Vrakozs az rsi adatra.                
         WR_DATA_WAIT: if (s_axi_wvalid)
                       begin
                          wr_state <= WR_EXECUTE;
                          wr_data  <= s_axi_wdata;
                          wr_strb  <= s_axi_wstrb;
                       end
                       else
                          wr_state <= WR_DATA_WAIT;

         //Az rsi mvelet vgrehajtsa.
         WR_EXECUTE  : wr_state <= WR_RESPONSE;

         //A nyugtzs elkldse.
         WR_RESPONSE : if (s_axi_bready)
                          wr_state <= WR_ADDR_WAIT;
                       else
                          wr_state <= WR_RESPONSE;
         
         //rvnytelen llapotok.
         default     : wr_state <= WR_ADDR_WAIT;
      endcase
end

//Az rsi cm csatorna READY jelzsnek ellltsa.
assign s_axi_awready = (wr_state == WR_ADDR_WAIT);
//Az rsi adat csatorna READY jelzsnek ellltsa.
assign s_axi_wready  = (wr_state == WR_DATA_WAIT);
//Az rsi vlasz csatorna VALID jelzsnek ellltsa.
assign s_axi_bvalid  = (wr_state == WR_RESPONSE);
//Mindog OKAY (00) nyugtt kldnk.
assign s_axi_bresp   = 2'b00;

//A regiszerek rs engedlyez jelnek ellltsa.
assign wr_en = (wr_state == WR_EXECUTE);


//******************************************************************************
//* Olvassi llapotgp.                                                       *
//******************************************************************************
localparam RD_ADDR_WAIT = 2'd0;
localparam RD_EXECUTE   = 2'd1;
localparam RD_SEND_DATA = 2'd2;

reg [1:0] rd_state;

always @(posedge clk)
begin
   if (rst)
      rd_state <= RD_ADDR_WAIT;
   else
      case (rd_state)
         //Vraozs az olvassi cmre.
         RD_ADDR_WAIT: if (s_axi_arvalid)
                       begin
                          rd_state <= RD_EXECUTE;
                          rd_addr  <= s_axi_araddr;
                       end
                       else
                          rd_state <= RD_ADDR_WAIT;

         //Az olvassi mvelet vgrehajtsa.
         RD_EXECUTE  : begin
                          rd_state    <= RD_SEND_DATA;
                          s_axi_rdata <= rd_data;
                       end

         //A beolvasott adat elkldse.
         RD_SEND_DATA: if (s_axi_rready)
                       begin
                          rd_state    <= RD_ADDR_WAIT;
                          s_axi_rdata <= 32'd0;
                       end
                       else
                          rd_state <= RD_SEND_DATA;
      endcase
end

//Az olvassi cm csatorna READY jelzsnek ellltsa.
assign s_axi_arready = (rd_state == RD_ADDR_WAIT);
//Az olvassi adat csatorna VALID jelzsnek ellltsa.
assign s_axi_rvalid  = (rd_state == RD_SEND_DATA);
//Mindog OKAY (00) nyugtt kldnk.
assign s_axi_rresp   = 2'b00;

//A regiszerek olvass engedlyez jelnek ellltsa.
assign rd_en = (rd_state == RD_EXECUTE);

endmodule

`default_nettype wire
