`timescale 1ns / 1ps
`default_nettype none

//******************************************************************************
//* UART perifria.                                                            *
//******************************************************************************
module uart_top(
   //rajel s reset.
   input  wire        s_axi_aclk,
   input  wire        s_axi_aresetn,
   
   //AXI4-Lite rsi cm csatorna.
   input  wire [4:0]  s_axi_awaddr,
   input  wire        s_axi_awvalid,
   output wire        s_axi_awready,
   
   //AXI4-Lite rsi adat csatorna.
   input  wire [31:0] s_axi_wdata,
   input  wire [3:0]  s_axi_wstrb,
   input  wire        s_axi_wvalid,
   output wire        s_axi_wready,
   
   //AXI4-Lite rsi vlasz csatorna.
   output wire [1:0]  s_axi_bresp,
   output wire        s_axi_bvalid,
   input  wire        s_axi_bready,
   
   //AXI4-Lite olvassi cm csatorna.
   input  wire [4:0]  s_axi_araddr,
   input  wire        s_axi_arvalid,
   output wire        s_axi_arready,
   
   //AXI4-Lite olvassi adat csatorna.
   output wire [31:0] s_axi_rdata,
   output wire [1:0]  s_axi_rresp,
   output wire        s_axi_rvalid,
   input  wire        s_axi_rready,
   
   //UART adatvonalak.
   input  wire        uart_rxd,
   output wire        uart_txd,
   
   //Megszaktskr kimenet.
   output wire        irq
);

//******************************************************************************
//* Bels rajel s reset jelek.                                               *
//******************************************************************************
wire clk =  s_axi_aclk;
wire rst = ~s_axi_aresetn;


//******************************************************************************
//* AXI4-Lite busz interfsz.                                                  *
//******************************************************************************
wire [4:0]  wr_addr;                   //rsi cm
wire        wr_en;                     //rs engedlyez jel
wire [31:0] wr_data;                   //rsi adat
wire [3:0]  wr_strb;                   //Bjt engedlyez jelek  
wire [4:0]  rd_addr;                   //Olvassi cm
wire        rd_en;                     //Olvass engedlyez jel
wire [31:0] rd_data;                   //Olvassi adat
   
axi4_lite_if #(
   //A hasznlt cmbitek szma.
   .ADDR_BITS(5)
) axi4_lite_if_i (
   //rajel s reset.
   .clk(clk),                          //Rendszerrajel
   .rst(rst),                          //Aktv magas szinkron reset
   
   //AXI4-Lite rsi cm csatorna.
   .s_axi_awaddr(s_axi_awaddr),
   .s_axi_awvalid(s_axi_awvalid),
   .s_axi_awready(s_axi_awready),
   
   //AXI4-Lite rsi adat csatorna.
   .s_axi_wdata(s_axi_wdata),
   .s_axi_wstrb(s_axi_wstrb),
   .s_axi_wvalid(s_axi_wvalid),
   .s_axi_wready(s_axi_wready),
   
   //AXI4-Lite rsi vlasz csatorna.
   .s_axi_bresp(s_axi_bresp),
   .s_axi_bvalid(s_axi_bvalid),
   .s_axi_bready(s_axi_bready),
   
   //AXI4-Lite olvassi cm csatorna.
   .s_axi_araddr(s_axi_araddr),
   .s_axi_arvalid(s_axi_arvalid),
   .s_axi_arready(s_axi_arready),
   
   //AXI4-Lite olvassi adat csatorna.
   .s_axi_rdata(s_axi_rdata),
   .s_axi_rresp(s_axi_rresp),
   .s_axi_rvalid(s_axi_rvalid),
   .s_axi_rready(s_axi_rready),
   
   //Regiszter rsi interfsz.
   .wr_addr(wr_addr),                  //rsi cm
   .wr_en(wr_en),                      //rs engedlyez jel
   .wr_data(wr_data),                  //rsi adat
   .wr_strb(wr_strb),                  //Bjt engedlyez jelek
    
   //Regiszter olvassi interfsz.
   .rd_addr(rd_addr),                  //Olvassi cm
   .rd_en(rd_en),                      //Olvass engedlyez jel
   .rd_data(rd_data)                   //Olvassi adat
);


//******************************************************************************
//* Az UART funkcit megvalstsa.                                            *
//******************************************************************************
uart uart_i(
   //rajel s reset.
   .clk(clk),                          //Rendszerrajel
   .rst(rst),                          //Aktv magas szinkron reset
   
   //Regiszter rsi interfsz.
   .wr_addr(wr_addr),                  //rsi cm
   .wr_en(wr_en),                      //rs engedlyez jel
   .wr_data(wr_data),                  //rsi adat
   .wr_strb(wr_strb),                  //Bjt engedlyez jelek
    
   //Regiszter olvassi interfsz.
   .rd_addr(rd_addr),                  //Olvassi cm
   .rd_en(rd_en),                      //Olvass engedlyez jel
   .rd_data(rd_data),                  //Olvassi adat
   
   //UART adatvonalak.
   .uart_rxd(uart_rxd),                //Soros bemenet
   .uart_txd(uart_txd),                //Soros kimenet
   
   //Megszaktskr kimenet.
   .irq(irq)
);


endmodule

`default_nettype wire
