`timescale 1ns / 1ps
`default_nettype none

module uart_top_test;

//Inputs
reg         s_axi_aclk;
reg         s_axi_aresetn;
reg  [4:0]  s_axi_awaddr;
reg         s_axi_awvalid;
reg  [31:0] s_axi_wdata;
reg  [3:0]  s_axi_wstrb;
reg         s_axi_wvalid;
reg         s_axi_bready;
reg [4:0]   s_axi_araddr;
reg         s_axi_arvalid;
reg         s_axi_rready;
reg         uart_rxd;

//Outputs
wire        s_axi_awready;
wire        s_axi_wready;
wire [1:0]  s_axi_bresp;
wire        s_axi_bvalid;
wire        s_axi_arready;
wire [31:0] s_axi_rdata;
wire [1:0]  s_axi_rresp;
wire        s_axi_rvalid;
wire        uart_txd;
wire        irq;

//Instantiate the Unit Under Test (UUT)
uart_top uut (
   .s_axi_aclk(s_axi_aclk), 
   .s_axi_aresetn(s_axi_aresetn), 
   .s_axi_awaddr(s_axi_awaddr), 
   .s_axi_awvalid(s_axi_awvalid), 
   .s_axi_awready(s_axi_awready), 
   .s_axi_wdata(s_axi_wdata), 
   .s_axi_wstrb(s_axi_wstrb), 
   .s_axi_wvalid(s_axi_wvalid), 
   .s_axi_wready(s_axi_wready), 
   .s_axi_bresp(s_axi_bresp), 
   .s_axi_bvalid(s_axi_bvalid), 
   .s_axi_bready(s_axi_bready), 
   .s_axi_araddr(s_axi_araddr), 
   .s_axi_arvalid(s_axi_arvalid), 
   .s_axi_arready(s_axi_arready), 
   .s_axi_rdata(s_axi_rdata), 
   .s_axi_rresp(s_axi_rresp), 
   .s_axi_rvalid(s_axi_rvalid), 
   .s_axi_rready(s_axi_rready), 
   .uart_rxd(uart_rxd), 
   .uart_txd(uart_txd), 
   .irq(irq)
);

//AXI4-Lite 32 bites rsi taszk.
task axi_write32(input [31:0] addr, input [31:0] data, output [1:0] resp);
begin
   addr = addr & 32'hffff_fffc;
   @(posedge s_axi_aclk);
   $display("%t: AXI write - address=0x%h, data=0x%h", $time, addr, data);
   
   fork
      //rsi cm csatorna.
      begin
         #1 s_axi_awaddr  = addr;
         s_axi_awvalid = 1'b1;
         wait (s_axi_awready == 1'b1);
         @(posedge s_axi_aclk);
         #1 s_axi_awvalid = 1'b0;        
      end
      
      //rsi adat csatorna.
      begin
         #1 s_axi_wdata = data;
         s_axi_wstrb = 4'b1111;
         s_axi_wvalid = 1'b1;
         wait (s_axi_wready == 1'b1);
         @(posedge s_axi_aclk);
         #1 s_axi_wvalid = 1'b0; 
      end
   join
   
   //rsi vlasz csatorna.
   #1 s_axi_bready = 1'b1;
   wait (s_axi_bvalid == 1'b1);
   $display("%t: AXI write - resp=0x%h", $time, s_axi_bresp);
   resp = s_axi_bresp;
   @(posedge s_axi_aclk);
   #1 s_axi_bready = 1'b0;
end
endtask

//AXI4-Lite 32 bites olvassi taszk.
task axi_read32(input [31:0] addr, output [31:0] data, output [1:0] resp);
begin
   addr = addr & 32'hffff_fffc;
   @(posedge s_axi_aclk);
   $display("%t: AXI read - address=0x%h", $time, addr);
   
   //Olvassi cm csatorna.
   #1 s_axi_araddr  = addr;
   s_axi_arvalid = 1'b1;
   wait (s_axi_arready == 1'b1);
   @(posedge s_axi_aclk);
   #1 s_axi_arvalid = 1'b0;
   
   //Olvassi adat csatorna.
   #1 s_axi_rready = 1'b1;
   wait (s_axi_rvalid == 1'b1);
   $display("%t: AXI read - resp=0x%h, data=0x%h", $time, s_axi_rresp, s_axi_rdata);
   data = s_axi_rdata;
   resp = s_axi_rresp;
   @(posedge s_axi_aclk);
   #1 s_axi_rready = 1'b0;
end
endtask

//Vrakozs, amg az adsi FIFO tele van (nem rhat).
task tx_fifo_wait;
integer fifo_status;
integer resp;
begin
   //A FIFO sttusz beolvassa.
   fifo_status = 32'h00;
   while ((fifo_status & 32'h01) == 0)
   begin
      axi_read32(32'h0c, fifo_status, resp);
   end
end
endtask

//Vrakozs, amg a vteli FIFO res (nem olvashat).
task rx_fifo_wait;
integer fifo_status;
integer resp;
begin
   //A FIFO sttusz beolvassa.
   fifo_status = 32'h00;
   while ((fifo_status & 32'h04) == 0)
   begin
      axi_read32(32'h0c, fifo_status, resp);
   end
end
endtask

integer resp;
integer data;

initial
begin
   // Initialize Inputs
   s_axi_aclk = 0;
   s_axi_aresetn = 0;
   s_axi_awaddr = 0;
   s_axi_awvalid = 0;
   s_axi_wdata = 0;
   s_axi_wstrb = 0;
   s_axi_wvalid = 0;
   s_axi_bready = 0;
   s_axi_araddr = 0;
   s_axi_arvalid = 0;
   s_axi_rready = 0;
   uart_rxd = 0;

   // Wait 100 ns for global reset to finish
   #100;

   // Add stimulus here
   #10;
   s_axi_aresetn = 1'b1;
   #10;
   
   //Az adattviteli sebessg belltsa:
   //- 1 bitid 32 rendszerrajel peridus.
   axi_write32(32'h04, 32'h01, resp);
   //Vonali kdols belltsa:
   //- 8 adatbit, nincs parits, 1 STOP bit.
   axi_write32(32'h08, 6'b0_00_0_11, resp);
   //Az ads s a vtel engedlyezse, a FIFO-k trlse.
   axi_write32(32'h00, 32'h0f, resp);
   //A vteli FIFO nem res megszakts engedlyezse.
   axi_write32(32'h14, 32'h04, resp);
   //Adatklds.
   tx_fifo_wait();
   axi_write32(32'h10, 32'h55, resp);
   //A vett adat beolvassa.
   rx_fifo_wait();
   axi_read32(32'h10, data, resp);
end

//rajel generlsa (100 MHz).
always
begin
   #5;
   s_axi_aclk <= ~s_axi_aclk;
end

//Az UART soros adat kimenett ksleltetve rktjl a soros adat bemenetre.
reg [9:0] txd_delay;

always @(posedge s_axi_aclk)
begin
   if (s_axi_aresetn == 0)
      txd_delay <= 10'b11_1111_1111;
   else
      txd_delay <= {txd_delay[8:0], uart_txd};
end

always @(*)
begin
   uart_rxd <= txd_delay[9];
end
      
endmodule

`default_nettype wire
